# Copyright 2024-present Coinbase Global, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import hmac
import hashlib
import base64
import requests
import time
import json
from typing import Optional, Dict, List
from prime_sdk.credentials import Credentials

DEFAULT_V1_API_BASE_URL = "https://api.prime.coinbase.com/v1"


class Client:
    def __init__(self, credentials: Credentials, http_client: Optional[requests.Session] = None):
        self.http_base_url = DEFAULT_V1_API_BASE_URL
        self.credentials = credentials
        self.http_client = http_client if http_client else requests.Session()

    def generate_headers(self, method: str, path: str, body: Optional[Dict] = None) -> Dict[str, str]:
        timestamp = str(int(time.time()))
        body_string = json.dumps(body) if body else ""
        message = f"{timestamp}{method}{path}{body_string}"
        signature = self.sign(message)

        return {
            "Accept": "application/json",
            "X-CB-ACCESS-KEY": self.credentials.access_key,
            "X-CB-ACCESS-PASSPHRASE": self.credentials.passphrase,
            "X-CB-ACCESS-SIGNATURE": signature,
            "X-CB-ACCESS-TIMESTAMP": timestamp,
        }

    def sign(self, message: str) -> str:
        h = hmac.new(self.credentials.signing_key.encode(), message.encode(), hashlib.sha256)
        return base64.b64encode(h.digest()).decode()

    def request(self, method: str, path: str, query: Optional[str] = "", body: Optional[Dict] = None,
                allowed_status_codes: Optional[List[int]] = None) -> requests.Response:
        if allowed_status_codes is None:
            allowed_status_codes = [200]
        full_path = f"{self.http_base_url}{path}"
        url = f"{full_path}?{query}" if query else full_path

        headers = self.generate_headers(method, f"/v1{path}", body)
        response = self.http_client.request(method, url, headers=headers, json=body)

        if response.status_code not in allowed_status_codes:
            try:
                error_details = response.json()
                error_message = error_details.get('message', response.text)
            except ValueError:
                error_message = response.text
            raise Exception(f"Request failed with status {response.status_code}: {error_message}")
        return response



import unittest
from prime_sdk.base_response import BaseResponse

class TestBaseResponse(unittest.TestCase):
    def test_initialization(self):
        response_data = {
            'field1': 'value1',
            'field2': 123,
            'field3': {'nested_field': 'nested_value'},
            'field4': [{'list_field': 'list_value'}]
        }
        base_response = BaseResponse(response=response_data)

        self.assertEqual(base_response.response, response_data)
        self.assertEqual(base_response.response['field1'], 'value1')
        self.assertEqual(base_response.response['field2'], 123)
        self.assertEqual(base_response.response['field3']['nested_field'], 'nested_value')
        self.assertEqual(base_response.response['field4'][0]['list_field'], 'list_value')

    def test_str_representation(self):
        response_data = {'field1': 'value1'}
        base_response = BaseResponse(response=response_data)
        self.assertEqual(str(base_response), '{"field1": "value1"}')

if __name__ == '__main__':
    unittest.main() 


# Copyright 2025-present Coinbase Global, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from dataclasses import dataclass
from prime_sdk.base_response import BaseResponse
from prime_sdk.client import Client
from typing import Optional, List
from datetime import datetime
from prime_sdk.credentials import Credentials
from prime_sdk.utils import PaginationParams, append_query_param, append_pagination_params, Pagination
from prime_sdk.enums import ActivityLevel
from prime_sdk.model import Activity


@dataclass
class ListEntityActivitiesRequest:
    entity_id: str
    activity_level: Optional[ActivityLevel] = None
    symbols: Optional[str] = None
    categories: Optional[str] = None
    statuses: Optional[str] = None
    start_time: Optional[datetime] = None
    end_time: Optional[datetime] = None
    pagination: Optional[PaginationParams] = None
    allowed_status_codes: Optional[List[int]] = None


@dataclass
class ListEntityActivitiesResponse(BaseResponse):
    activities: List[Activity] = None
    pagination: Pagination = None


class PrimeClient:
    def __init__(self, credentials: Credentials):
        self.client = Client(credentials)

    def list_entity_activities(self, request: ListEntityActivitiesRequest) -> ListEntityActivitiesResponse:
        path = f"/entities/{request.entity_id}/activities"

        query_params = append_query_param("", 'activity_level', request.symbols)
        query_params = append_query_param(query_params, 'symbols', request.symbols)
        query_params = append_query_param(query_params, 'categories', request.categories)
        query_params = append_query_param(query_params, 'statuses', request.statuses)

        if request.start_time:
            query_params = append_query_param(
                query_params,
                'start_time',
                request.start_time.isoformat() + 'Z')
        if request.end_time:
            query_params = append_query_param(
                query_params,
                'end_time',
                request.end_time.isoformat() + 'Z')

        query_params = append_pagination_params(query_params, request.pagination)

        response = self.client.request("GET", path, query=query_params, allowed_status_codes=request.allowed_status_codes)
        return ListEntityActivitiesResponse(response.json())



from prime_sdk.get_order import GetOrderResponse
from prime_sdk.model import Order


def test_get_order_response_parsing():
    mock_json = {
        "order": {
            "id": "order-123",
            "user_id": "user-abc",
            "portfolio_id": "portfolio-xyz",
            "product_id": "BTC-USD",
            "side": "BUY",
            "client_order_id": "client-order-1",
            "type": "LIMIT",
            "base_quantity": "1.0",
            "quote_value": "60000.00",
            "limit_price": "60000.00",
            "start_time": "2024-01-01T00:00:00Z",
            "expiry_time": "2024-01-01T01:00:00Z",
            "status": "FILLED",
            "time_in_force": "GTC",
            "created_at": "2024-01-01T00:00:00Z",
            "filled_quantity": "1.0",
            "filled_value": "60000.00",
            "average_filled_price": "60000.00",
            "commission": "0.00",
            "exchange_fee": "0.00",
            "historical_pov": "100",
            "stop_price": "0.00",
            "net_average_filled_price": "60000.00",
            "user_context": "some-context",
            "client_product_id": "BTC-USD"
        }
    }

    response = GetOrderResponse(response=mock_json)

    assert isinstance(response.response, dict)
    assert "order" in response.response
    assert isinstance(response.order, Order)

    assert response.order.id == "order-123"
    assert response.order.side == "BUY"
    assert response.order.status == "FILLED"
    assert response.order.product_id == "BTC-USD"



# Copyright 2025-present Coinbase Global, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from dataclasses import dataclass, asdict
from prime_sdk.base_response import BaseResponse
from prime_sdk.client import Client
from typing import List, Optional
from prime_sdk.credentials import Credentials


@dataclass
class Inputs:
    amount: str


@dataclass
class CreateStakeRequest:
    portfolio_id: str
    wallet_id: str
    idempotency_key: str
    inputs: Optional[Inputs] = None
    allowed_status_codes: Optional[List[int]] = None


@dataclass
class CreateStakeResponse(BaseResponse):
    wallet_id: str = None
    transaction_id: str = None
    activity_id: str = None


class PrimeClient:
    def __init__(self, credentials: Credentials):
        self.client = Client(credentials)
        
    def create_stake(self, request: CreateStakeRequest) -> CreateStakeResponse:
        path = f"/portfolios/{request.portfolio_id}/wallets/{request.wallet_id}/staking/initiate"
        body = {k: v for k, v in asdict(request).items() if v is not None}
        
        if request.inputs:
            body["inputs"] = asdict(request.inputs)

        response = self.client.request("POST", path, body=body, allowed_status_codes=request.allowed_status_codes)
        return CreateStakeResponse(response.json())



# Copyright 2024-present Coinbase Global, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from dataclasses import dataclass
import os
import json


@dataclass
class Credentials:
    access_key: str
    passphrase: str
    signing_key: str
    portfolio_id: str
    entity_id: str
    svc_account_id: str

    @staticmethod
    def from_json(data: str) -> 'Credentials':
        credentials_dict = json.loads(data)
        return Credentials(
            access_key=credentials_dict['accessKey'],
            passphrase=credentials_dict['passphrase'],
            signing_key=credentials_dict['signingKey'],
            portfolio_id=credentials_dict['portfolioId'],
            entity_id=credentials_dict['entityId'],
            svc_account_id=credentials_dict['svcAccountId']
        )

    @staticmethod
    def from_env(variable_name: str) -> 'Credentials':
        env_var = os.getenv(variable_name)
        if not env_var:
            raise EnvironmentError(
                f"{variable_name} not set as environment variable")
        return Credentials.from_json(env_var)



# Copyright 2025-present Coinbase Global, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from dataclasses import dataclass, asdict
from prime_sdk.base_response import BaseResponse
from prime_sdk.client import Client
from typing import List, Optional
from prime_sdk.credentials import Credentials
from prime_sdk.enums import SizeType
from warnings import warn


@dataclass
class AllocationLeg:
    allocation_leg_id: str
    destination_portfolio_id: str
    amount: str


@dataclass
class CreatePortfolioNetAllocationsRequest:
    allocation_id: str
    source_portfolio_id: str
    product_id: str
    order_ids: List[str]
    allocation_legs: List[AllocationLeg]
    size_type: SizeType
    remainder_destination_portfolio_id: str
    allowed_status_codes: Optional[List[int]] = None


@dataclass
class CreatePortfolioNetAllocationsResponse(BaseResponse):
    success: bool = None
    netting_id: str = None
    buy_allocation_id: str = None
    sell_allocation_id: str = None
    failure_reason: str = None


class PrimeClient:
    def __init__(self, credentials: Credentials):
        self.client = Client(credentials)

    def create_portfolio_net_allocations(self, request: CreatePortfolioNetAllocationsRequest) -> CreatePortfolioNetAllocationsResponse:
        path = "/allocations/net"

        body = {k: v for k, v in asdict(request).items() if v is not None}
        if request.allocation_legs:
            body["allocation_legs"] = [asdict(leg) for leg in request.allocation_legs]

        response = self.client.request("POST", path, body=body, allowed_status_codes=request.allowed_status_codes)
        return CreatePortfolioNetAllocationsResponse(response.json())



# Copyright 2024-present Coinbase Global, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
from prime_sdk.credentials import Credentials
from prime_sdk.list_portfolios import PrimeClient, ListPortfoliosRequest


def main():
    credentials = Credentials.from_env("PRIME_CREDENTIALS")
    prime_client = PrimeClient(credentials)

    request = ListPortfoliosRequest()
    try:
        response = prime_client.list_portfolios(request)
        print(response)
    except Exception as e:
        print(f"failed to list portfolios: {e}")


if __name__ == "__main__":
    main()



# Copyright 2025-present Coinbase Global, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from enum import Enum

class WalletType(Enum):
    VAULT   = "VAULT"
    TRADING = "TRADING"
    ONCHAIN = "ONCHAIN"
    OTHER   = "WALLET_TYPE_OTHER"


class WalletDepositType(Enum):
    WIRE   = "WIRE"
    SWIFT  = "SWIFT"
    CRYPTO = "CRYPTO"
    SEPA   = "SEPA"


class BalanceType(Enum):
    TRADING = "TRADING_BALANCES"
    VAULT   = "VAULT_BALANCES"
    TOTAL   = "TOTAL_BALANCES"
    UNKNOWN_BALANCE_TYPE = "UNKNOWN_BALANCE_TYPE"
    PRIME_CUSTODY_BALANCES = "PRIME_CUSTODY_BALANCES"


class NetworkType(Enum):
    NETWORK_TYPE_UNSPECIFIED = "NETWORK_TYPE_UNSPECIFIED"
    NETWORK_TYPE_EVM = "NETWORK_TYPE_EVM"
    NETWORK_TYPE_SOLANA = "NETWORK_TYPE_SOLANA"


class OrderSide(Enum):
    BUY  = "BUY"
    SELL = "SELL"


class OrderType(Enum):
    MARKET = "MARKET"
    LIMIT  = "LIMIT"
    TWAP   = "TWAP"
    BLOCK  = "BLOCK"
    VWAP   = "VWAP"
    STOP_LIMIT = "STOP_LIMIT"
    RFQ = "RFQ"


class TimeInForce(Enum):
    GOOD_UNTIL_DATE_TIME      = "GOOD_UNTIL_DATE_TIME"
    GOOD_UNTIL_CANCELLED       = "GOOD_UNTIL_CANCELLED"
    IMMEDIATE_OR_CANCEL        = "IMMEDIATE_OR_CANCEL"
    FILL_OR_KILL               = "FILL_OR_KILL"
    UNKNOWN_TIME_IN_FORCE      = "UNKNOWN_TIME_IN_FORCE"


class SizeType(Enum):
    ALLOCATION_SIZE_TYPE_UNKNOWN = "ALLOCATION_SIZE_TYPE_UNKNOWN"
    BASE = "BASE"
    QUOTE = "QUOTE"
    PERCENT = "PERCENT"


class ActivityLevel(Enum):
    ACTIVITY_LEVEL_ALL = "ACTIVITY_LEVEL_ALL"
    ACTIVITY_LEVEL_ENTITY = "ACTIVITY_LEVEL_ENTITY"
    ACTIVITY_LEVEL_PORTFOLIO = "ACTIVITY_LEVEL_PORTFOLIO"


class AggregationType(Enum):
    UNKNOWN_BALANCE_TYPE = "UNKNOWN_BALANCE_TYPE"
    TRADING_BALANCES = "TRADING_BALANCES"
    VAULT_BALANCES = "VAULT_BALANCES"
    TOTAL_BALANCES = "TOTAL_BALANCES"
    PRIME_CUSTODY_BALANCES = "PRIME_CUSTODY_BALANCES"
    UNIFIED_TOTAL_BALANCES = "UNIFIED_TOTAL_BALANCES"



# Copyright 2024-present Coinbase Global, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from dataclasses import dataclass
from prime_sdk.base_response import BaseResponse
from prime_sdk.client import Client
from typing import Optional, List
from prime_sdk.credentials import Credentials
from prime_sdk.utils import PaginationParams, append_query_param, append_pagination_params
from prime_sdk.model import Invoice


@dataclass
class ListInvoicesRequest:
    entity_id: str
    states: Optional[str] = None
    billing_year: Optional[int] = None
    billing_month: Optional[str] = None
    pagination: Optional[PaginationParams] = None
    allowed_status_codes: Optional[List[int]] = None


@dataclass
class ListInvoicesResponse(BaseResponse):
    invoices: List[Invoice] = None


class PrimeClient:
    def __init__(self, credentials: Credentials):
        self.client = Client(credentials)

    def list_invoices(self, request: ListInvoicesRequest) -> ListInvoicesResponse:
        path = f"/entities/{request.entity_id}/invoices"
        query_params = append_query_param("", 'states', request.states)
        query_params = append_query_param(query_params, 'billing_year', request.billing_year)
        query_params = append_query_param(query_params, 'billing_month', request.billing_month)
        query_params = append_pagination_params(query_params, request.pagination)
        response = self.client.request("GET", path, query=query_params, allowed_status_codes=request.allowed_status_codes)
        return ListInvoicesResponse(response.json())



# Copyright 2024-present Coinbase Global, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from dataclasses import dataclass, asdict
from typing import Optional, List, Union, Dict


@dataclass
class PaginationParams:
    cursor: str = ''
    limit: str = ''
    sort_direction: str = ''

    def to_dict(self) -> Dict[str, str]:
        return asdict(self)


def append_query_param(query_params: str, key: str, value: Optional[Union[str, List[str]]]) -> str:
    if value:
        if isinstance(value, list):
            for v in value:
                query_params = f"{query_params}&{key}={v}" if query_params else f"{key}={v}"
        else:
            query_params = f"{query_params}&{key}={value}" if query_params else f"{key}={value}"
    return query_params


def append_pagination_params(query_params: str, pagination: Optional[PaginationParams]) -> str:
    if pagination:
        query_params = append_query_param(query_params, 'cursor', pagination.cursor)
        query_params = append_query_param(query_params, 'limit', pagination.limit)
        query_params = append_query_param(query_params, 'sort_direction', pagination.sort_direction)
    return query_params


@dataclass
class Pagination:
    next_cursor: str = ''
    sort_direction: str = ''
    has_next: bool = False

    def to_dict(self) -> Dict[str, str]:
        return asdict(self)



# Copyright 2024-present Coinbase Global, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from dataclasses import dataclass, asdict
from prime_sdk.base_response import BaseResponse
from prime_sdk.client import Client
from typing import Optional, List
from prime_sdk.credentials import Credentials
from prime_sdk.model import Blockchain


@dataclass
class PaymentMethod:
    payment_method_id: str


@dataclass
class BlockchainAddress:
    address: str
    account_identifier: Optional[str] = None
    allowed_status_codes: Optional[List[int]] = None


@dataclass
class CreateWithdrawalRequest:
    portfolio_id: str
    wallet_id: str
    amount: str
    destination_type: str
    idempotency_key: str
    currency_symbol: str
    payment_method: Optional[PaymentMethod] = None
    blockchain_address: Optional[BlockchainAddress] = None
    allowed_status_codes: Optional[List[int]] = None


@dataclass
class CreateWithdrawalResponse(BaseResponse):
    activity_id: str = None
    approval_url: str = None
    symbol: str = None
    amount: str = None
    fee: str = None
    destination_type: str = None
    source_type: str = None
    blockchain_destination: Blockchain = None
    blockchain_source: Blockchain = None
    transaction_id: str = None


class PrimeClient:
    def __init__(self, credentials: Credentials):
        self.client = Client(credentials)
        
    def create_withdrawal(self, request: CreateWithdrawalRequest) -> CreateWithdrawalResponse:
        path = f"/portfolios/{request.portfolio_id}/wallets/{request.wallet_id}/withdrawals"

        body = {k: v for k, v in asdict(request).items() if v is not None}

        if request.payment_method:
            body["payment_method"] = asdict(request.payment_method)

        if request.blockchain_address:
            body["blockchain_address"] = asdict(request.blockchain_address)

        response = self.client.request("POST", path, body=body, allowed_status_codes=request.allowed_status_codes)
        return CreateWithdrawalResponse(response.json())



# Copyright 2024-present Coinbase Global, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import json
import dataclasses
from dataclasses import dataclass, field
from typing import Any, Dict, get_origin, get_args, get_type_hints


@dataclass
class BaseResponse:
    response: Dict[str, Any] = field(default_factory=dict)

    def __init__(self, response: Dict[str, Any] = None, **kwargs):
        object.__setattr__(self, "response", response or {})

        for field_name in self.__dataclass_fields__:
            if field_name == "response":
                continue
            if field_name in kwargs:
                object.__setattr__(self, field_name, kwargs[field_name])

    def __post_init__(self):
        child_cls = type(self)

        type_hints = get_type_hints(child_cls)

        for field_name, real_type in type_hints.items():
            if field_name == "response":
                continue

            current_val = getattr(self, field_name, None)
            if current_val is not None:
                continue

            raw_value = self.response.get(field_name)
            if raw_value is None:
                continue

            if dataclasses.is_dataclass(real_type) and isinstance(raw_value, dict):
                parsed_obj = real_type(**raw_value)
                object.__setattr__(self, field_name, parsed_obj)
                continue

            if get_origin(real_type) is list:
                (inner_type,) = get_args(real_type)
                if dataclasses.is_dataclass(inner_type) and isinstance(raw_value, list):
                    parsed_list = [inner_type(**item) for item in raw_value]
                    object.__setattr__(self, field_name, parsed_list)
                    continue

            object.__setattr__(self, field_name, raw_value)

    def __str__(self) -> str:
        return json.dumps(self.response)

    def __repr__(self) -> str:
        return self.__str__()



# Copyright 2025-present Coinbase Global, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from dataclasses import dataclass
from typing import List
from prime_sdk.enums import NetworkType


@dataclass
class Address:
    name: str
    address: str
    chain_ids: List[str]


@dataclass
class AddressGroup:
    id: str
    name: str
    network_type: NetworkType
    addresses: List[Address]
    added_at: str


@dataclass
class Portfolio:
    id: str
    name: str
    entity_id: str
    organization_id: str
    entity_name: str


@dataclass
class PortfolioUser:
    id: str
    name: str
    email: str
    portfolio_id: str
    entity_id: str
    role: str


@dataclass
class User:
    id: str
    name: str
    email: str
    entity_id: str
    role: str


@dataclass
class Network:
    id: str
    type: str


@dataclass
class Blockchain:
    address: str
    account_identifier: str
    network: Network


@dataclass
class AssetNetwork:
    network: Network
    name: str
    max_decimals: str
    default: bool
    trading_supported: bool
    vault_supported: bool
    prime_custody_supported: bool
    destination_tag_required: bool
    network_link: str


@dataclass
class Asset:
    name: str
    symbol: str
    decimal_precision: str
    trading_supported: bool
    explorer_url: str
    networks: List[AssetNetwork]
  

@dataclass
class ConsensusMetadata:
    approval_deadline: str
    has_passed_consensus: bool


@dataclass
class TransactionsMetadata:
    consensus: ConsensusMetadata


@dataclass
class AccountMetadata:
    consensus: ConsensusMetadata


@dataclass
class UserAction:
    action: str
    user_id: str
    timestamp: str


@dataclass
class Activity:
    id: str
    reference_id: str
    category: str
    type: str
    secondary_type: str
    status: str
    created_by: str
    title: str
    description: str
    user_actions: List[UserAction]
    transactions_metadata: TransactionsMetadata = None
    account_metadata: AccountMetadata = None
    symbols: List[str] = None
    created_at: str = None
    updated_at: str = None
    hierarchy_type: str = None


@dataclass
class ActivityResponse:
    activity: Activity


@dataclass
class Add:
    id: str
    name: str
    avatar_url: str


@dataclass
class Address:
    id: str
    currency_symbol: str
    name: str
    address: str
    account_identifier: str
    account_identifier_name: str
    state: str
    explorer_link: str
    last_used_at: str
    added_at: str
    added_by: Add
    type: str
    counterparty_id: str


@dataclass
class Destination:
    leg_id: str
    portfolio_id: str
    allocation_base: str
    allocation_quote: str
    fees_allocated_leg: str


@dataclass
class Allocation:
    root_id: str
    reversal_id: str
    allocation_completed_at: str
    user_id: str
    product_id: str
    side: str
    avg_price: str
    base_quantity: str
    quote_value: str
    fees_allocated: str
    status: str
    source: str
    order_ids: List[str]
    destinations: List[Destination]
    netting_id: str


@dataclass
class Details:
    id: str
    symbol: str
    payment_method_type: str
    name: str
    account_number: str
    bank_code: str
    

@dataclass
class Position:
    product_id: str
    side: str
    number_of_contracts: str
    daily_realized_pnl: str
    unrealized_pnl: str
    current_price: str
    avg_entry_price: str
    expiration_time: str


@dataclass
class Order:
    id: str
    user_id: str
    portfolio_id: str
    product_id: str
    side: str
    client_order_id: str
    type: str
    base_quantity: str
    quote_value: str
    limit_price: str
    start_time: str
    expiry_time: str
    status: str
    time_in_force: str
    created_at: str
    filled_quantity: str
    filled_value: str
    average_filled_price: str
    commission: str
    exchange_fee: str
    historical_pov: str
    stop_price: str
    net_average_filled_price: str
    user_context: str
    client_product_id: str


@dataclass
class Commission:
    type: str
    rate: str
    trading_volume: str


@dataclass
class AmountDue:
    currency: str
    amount: str
    due_date: str

@dataclass
class PostTradeCredit:
    portfolio_id: str
    currency: str
    limit: str
    utilized: str
    available: str
    frozen: str
    frozen_reason: str
    amounts_due: List[AmountDue]
    enabled: str
    adjusted_credit_utilized: str
    adjusted_portfolio_equity: str


@dataclass
class TransferLocation:
    type: str
    value: str


@dataclass
class EstimatedNetworkFees:
    lower_bound: str
    upper_bound: str


@dataclass
class Collection:
    name: str


@dataclass
class Item:
    name: str


@dataclass
class AssetChange:
    type: str
    symbol: str
    amount: str
    collection: Collection
    item: Item


@dataclass
class MatchMetadata:
    reference_id: str
    settlement_date: str


@dataclass
class Web3TransactionMetadata:
    label: str
    confirmed_asset_changes: List[AssetChange]


@dataclass
class TransactionMetadata:
    match_metadata: MatchMetadata
    web3_transaction_metadata: Web3TransactionMetadata


@dataclass
class RiskAssessment:
    compliance_risk_detected: bool
    security_risk_detected: bool


@dataclass
class OnchainDetails:
    signed_transaction: str
    risk_assessment: RiskAssessment
    chain_id: str
    nonce: str
    replaced_transaction_id: str
    destination_address: str
    skip_broadcast: bool
    failure_reason: str
    signing_status: str


@dataclass
class Transaction:
    id: str
    wallet_id: str
    portfolio_id: str
    type: str
    status: str
    symbol: str
    created_at: str
    completed_at: str
    amount: str
    transfer_from: TransferLocation
    transfer_to: TransferLocation
    network_fees: str
    fees: str
    fee_symbol: str
    blockchain_ids: List[str]
    transaction_id: str
    destination_symbol: str
    estimated_network_fees: EstimatedNetworkFees
    network: str
    estimated_asset_changes: List[AssetChange]
    metadata: TransactionMetadata
    idempotency_key: str
    onchain_details: OnchainDetails
    network_info: Network


@dataclass
class Balance:
    symbol: str
    amount: str
    holds: str
    bonded_amount: str
    reserved_amount: str
    unbonding_amount: str
    unvested_amount: str
    pending_rewards_amount: str
    past_rewards_amount: str
    bondable_amount: str
    withdrawable_amount: str
    fiat_amount: str
    unbondable_amount: str


@dataclass
class CryptoInstructions:
    id: str
    name: str
    type: str
    address: str
    account_identifier: str
    account_identifier_name: str
    network: Network


@dataclass
class FiatInstructions:
    id: str
    name: str
    type: str
    account_number: str
    routing_number: str
    reference_code: str


@dataclass
class Instructions:
    crypto_instructions: CryptoInstructions
    fiat_instructions: FiatInstructions


@dataclass
class Wallet:
    id: str
    name: str
    symbol: str
    type: str
    created_at: str
    address: str
    visibility: str
    network: Network


@dataclass
class RequestedAmount:
    currency: str
    amount: str


@dataclass
class Sweeps:
    id: str
    requested_amount: RequestedAmount
    should_sweep_all: bool
    status: str
    scheduled_time: str


@dataclass
class InvoiceItem:
    description: str
    currency_symbol: str
    invoice_type: str
    rate: float
    quantity: float
    price: float
    average_auc: float
    total: float


@dataclass
class Invoices:
    id: str
    billing_month: int
    billing_year: int
    due_date: str
    invoice_number: str
    state: str
    usd_amount_paid: float
    usd_amount_owed: float
    invoice_items: List[InvoiceItem]


@dataclass
class Fill:
    id: str
    order_id: str
    product_id: str
    client_product_id: str
    side: str
    filled_quantity: str
    filled_value: str
    price: str
    time: str
    commission: str
    venue: str


@dataclass
class BalanceWithHolds:
    total: str
    holds: str


@dataclass
class RfqProductDetails:
    tradable: bool
    min_notional_size: str
    max_notional_size: str


@dataclass
class Product:
    id: str
    base_increment: str
    quote_increment: str
    base_min_size: str
    quote_min_size: str
    base_max_size: str
    quote_max_size: str
    permissions: List[str]
    price_increment: str
    rfq_product_details: RfqProductDetails


@dataclass
class DefiBalances:
    network: str
    protocol: str
    net_usd_value: str


@dataclass
class OnchainBalance:
    asset: Asset
    amount: str
    visibility_status: str


@dataclass
class MarketRate:
    symbol: str
    rate: str


@dataclass
class AssetBalance:
    portfolio_id: str
    symbol: str
    amount: str
    notional_amount: str
    conversion_rate: str


@dataclass
class TfLoan:
    portfolio_id: str
    symbol: str
    amount: str
    notional_amount: str
    due_date: str


@dataclass
class PmLoan:
    portfolio_id: str
    symbol: str
    amount: str
    notional_amount: str
    due_date: str


@dataclass
class ShortCollateral:
    portfolio_id: str
    symbol: str
    amount: str
    notional_amount: str
    due_date: str


@dataclass
class PortfolioStressTriggered:
    amount: str
    add_on_type: str


@dataclass
class PmAssetInfo:
    symbol: str
    amount: str
    price: str
    notional_amount: str
    asset_tier: str
    margin_eligible: bool
    base_margin_requirement: str
    base_margin_requirement_notional: str
    adv_30d: str
    hist_5d_vol: str
    hist_30d_vol: str
    hist_90d_vol: str
    volatility_addon: str
    liquidity_addon: str
    total_position_margin: str
    short_nominal: str
    long_nominal: str


@dataclass
class MarginSummary:
    entity_id: str
    margin_equity: str
    margin_requirement: str
    excess_deficit: str
    pm_credit_consumed: str
    tf_credit_limit: str
    tf_credit_consumed: str
    tf_adjusted_asset_value: str
    tf_adjusted_liability_value: str
    tf_adjusted_credit_consumed: str
    tf_adjusted_equity: str
    frozen: bool
    frozen_reason: str
    tf_enabled: bool
    pm_enabled: bool
    market_rates: List[MarketRate]
    asset_balances: List[AssetBalance]
    tf_loans: List[TfLoan]
    pm_loans: List[PmLoan]
    short_collateral: List[ShortCollateral]
    gross_market_value: str
    net_market_value: str
    long_market_value: str
    non_marginable_long_market_value: str
    short_market_value: str
    gross_leverage: str
    net_exposure: str
    portfolio_stress_triggered: PortfolioStressTriggered
    pm_asset_info: List[PmAssetInfo]
    pm_credit_limit: str
    pm_margin_limit: str
    pm_margin_consumed: str


@dataclass
class MarginCall:
    id: str
    initial_notional_amount: str
    outstanding_notional_amount: str
    created_at: str
    due_at: str


@dataclass
class WithdrawalPower:
    symbol: str
    amount: str


@dataclass
class BuyingPower:
    portfolio_id: str
    base_currency: str
    quote_currency: str
    base_buying_power: str
    quote_buying_power: str


@dataclass
class ConversionDetail:
    symbol: str
    tf_balance: str
    notional_tf_balance: str
    converted_balance: str
    notional_converted_balance: str
    interest_rate: str
    conversion_rate: str


@dataclass
class ShortCollateral:
    old_balance: str
    new_balance: str
    loan_interest_rate: str
    collateral_interest_rate: str


@dataclass
class Conversion:
    conversion_details: List[ConversionDetail]
    short_collateral: ShortCollateral
    conversion_datetime: str
    portfolio_id: str


@dataclass
class Accrual:
    accrual_id: str
    date: str
    portfolio_id: str
    symbol: str
    loan_type: str
    interest_rate: str
    nominal_accrual: str
    notional_accrual: str
    conversion_rate: str
    loan_amount: str
    benchmark: str
    benchmark_rate: str
    spread: str
    rate_type: str
    loan_amount_notional: str
    nominal_open_borrow_sod: str
    notional_open_bnorrow_sod: str


@dataclass
class Reference:
    id: str
    type: str


@dataclass
class Position:
    symbol: str
    long: str
    short: str
    reference: Reference


@dataclass
class MarginSummaryRecord:
    conversion_datetime: str
    conversion_date: str
    margin_summary: MarginSummary


@dataclass
class Balance:
    symbol: str
    long_amount: str
    long_notional: str
    short_amount: str
    short_notional: str


@dataclass
class LocateAvailability:
    symbol: str
    quantity: str
    rate: str


@dataclass
class Locate:
    locate_id: str
    entity_id: str
    portfolio_id: str
    symbol: str
    requested_amount: str
    interest_rate: str
    status: str
    approved_amount: str
    conversion_date: str
    created_at: str
    locate_date: str


@dataclass
class MarginCallRecord:
    margin_call_id: str
    initial_notional_amount: str
    outstanding_notional_amount: str
    created_at: str
    due_at: str


@dataclass
class MarginInformation:
    margin_call_records: List[MarginCallRecord]
    margin_summary: MarginSummary


@dataclass
class AmountDue:
    currency: str
    amount: str
    due_date: str


@dataclass
class PostTradeCredit:
    portfolio_id: str
    currency: str
    limit: str
    utilized: str
    available: str
    frozen: bool
    frozen_reason: str
    amounts_due: List[AmountDue]
    enabled: bool
    adjusted_credit_utilized: str
    adjusted_portfolio_equity: str


@dataclass
class Fee:
    symbol: str
    fee: str



from prime_sdk.list_portfolios import ListPortfoliosResponse
from prime_sdk.model import Portfolio


def test_list_portfolios_response_parsing():
    mock_json = {
        "portfolios": [
            {
                "id": "fake-portfolio-id-123",
                "name": "CryptoBalances",
                "entity_id": "fake-entity-id",
                "organization_id": "fake-org-id",
                "entity_name": "Sample Prime Entity"
            }
        ]
    }

    response = ListPortfoliosResponse(response=mock_json)

    assert isinstance(response.response, dict)
    assert isinstance(response.portfolios, list)
    assert len(response.portfolios) == 1
    assert isinstance(response.portfolios[0], Portfolio)

    portfolio = response.portfolios[0]
    assert portfolio.id == "fake-portfolio-id-123"
    assert portfolio.name == "CryptoBalances"
    assert portfolio.entity_id == "fake-entity-id"
    assert portfolio.organization_id == "fake-org-id"
    assert portfolio.entity_name == "Sample Prime Entity"



# Copyright 2024-present Coinbase Global, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from setuptools import setup, find_packages

setup(
    name="prime-sdk-py",
    version="0.4.1",
    packages=find_packages(),
    install_requires=[
        'requests',
    ],
    entry_points={
        'console_scripts': [
            'prime-sdk=prime_sdk.__main__:main',
        ],
    },
)
